@@ -1,6 +1,6 @@
 /*
  * filter_channelcopy.c -- copy one audio channel to another
- * Copyright (C) 2003-2004 Ushodaya Enterprises Limited
+ * Copyright (C) 2003-2012 Ushodaya Enterprises Limited
  * Author: Dan Dennedy <dan@dennedy.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -39,34 +39,90 @@ static int filter_get_audio( mlt_frame frame, void **buffer, mlt_audio_format *f
 
 	int from = mlt_properties_get_int( properties, "channelcopy.from" );
 	int to = mlt_properties_get_int( properties, "channelcopy.to" );
+	int swap = mlt_properties_get_int( properties, "channelcopy.swap" );
 
 	// Get the producer's audio
 	mlt_frame_get_audio( frame, buffer, format, frequency, channels, samples );
 
 	// Copy channels as necessary
+	if ( from != to)
 	switch ( *format )
 	{
 		case mlt_audio_s16:
 		{
 			int16_t *f = (int16_t*) *buffer + from;
 			int16_t *t = (int16_t*) *buffer + to;
+			int16_t x;
 			int i;
-			for ( i = 0; i < *samples; i++, f += *channels, t += *channels )
-				*t = *f;
+
+			if ( swap )
+				for ( i = 0; i < *samples; i++, f += *channels, t += *channels )
+				{
+					x = *t;
+					*t = *f;
+					*f = x;
+				}
+			else
+				for ( i = 0; i < *samples; i++, f += *channels, t += *channels )
+					*t = *f;
 			break;
 		}
 		case mlt_audio_s32:
 		{
 			int32_t *f = (int32_t*) *buffer + from * *samples;
 			int32_t *t = (int32_t*) *buffer + to * *samples;
-			memcpy( t, f, *samples * sizeof(int32_t) );
+
+			if ( swap )
+			{
+				int32_t *x = malloc( *samples * sizeof(int32_t) );
+				memcpy( x, t, *samples * sizeof(int32_t) );
+				memcpy( t, f, *samples * sizeof(int32_t) );
+				memcpy( f, x, *samples * sizeof(int32_t) );
+				free( x );
+			}
+			else
+			{
+				memcpy( t, f, *samples * sizeof(int32_t) );
+			}
+			break;
+		}
+		case mlt_audio_s32le:
+		case mlt_audio_f32le:
+		{
+			int32_t *f = (int32_t*) *buffer + from;
+			int32_t *t = (int32_t*) *buffer + to;
+			int32_t x;
+			int i;
+
+			if ( swap )
+				for ( i = 0; i < *samples; i++, f += *channels, t += *channels )
+				{
+					x = *t;
+					*t = *f;
+					*f = x;
+				}
+			else
+				for ( i = 0; i < *samples; i++, f += *channels, t += *channels )
+					*t = *f;
 			break;
 		}
 		case mlt_audio_float:
 		{
 			float *f = (float*) *buffer + from * *samples;
 			float *t = (float*) *buffer + to * *samples;
-			memcpy( t, f, *samples * sizeof(float) );
+
+			if ( swap )
+			{
+				float *x = malloc( *samples * sizeof(float) );
+				memcpy( x, t, *samples * sizeof(float) );
+				memcpy( t, f, *samples * sizeof(float) );
+				memcpy( f, x, *samples * sizeof(float) );
+				free( x );
+			}
+			else
+			{
+				memcpy( t, f, *samples * sizeof(float) );
+			}
 			break;
 		}
 		default:
@@ -88,6 +144,7 @@ static mlt_frame filter_process( mlt_filter this, mlt_frame frame )
 	// Propogate the parameters
 	mlt_properties_set_int( frame_props, "channelcopy.to", mlt_properties_get_int( properties, "to" ) );
 	mlt_properties_set_int( frame_props, "channelcopy.from", mlt_properties_get_int( properties, "from" ) );
+	mlt_properties_set_int( frame_props, "channelcopy.swap", mlt_properties_get_int( properties, "swap" ) );
 
 	// Override the get_audio method
 	mlt_frame_push_audio( frame, this );
@@ -109,6 +166,8 @@ mlt_filter filter_channelcopy_init( mlt_profile profile, mlt_service_type type,
 			mlt_properties_set_int( MLT_FILTER_PROPERTIES( this ), "to", atoi( arg ) );
 		else
 			mlt_properties_set_int( MLT_FILTER_PROPERTIES( this ), "to", 1 );
+		if ( strcmp(id, "channelswap") == 0 )
+			mlt_properties_set_int( MLT_FILTER_PROPERTIES( this ), "swap", 1 );
 	}
 	return this;
 }
